type proposition =
  | Var of string
  | Vrai
  | Faux
  | Neg of proposition
  | Implique of proposition * proposition
  | Et of proposition * proposition
  | Ou of proposition * proposition
;;

(*1*)
let prop_ex =
  let a = Var "A" and b = Var "B" and c = Var "C" in
    Ou (a,
        (Implique (Neg (b),
                   Et (a, c))))
;;

(*2*)
let rec string_of_prop p = match p with 
  | Var s -> s
  | Vrai -> "Vrai"
  | Faux -> "Faux"
  | Neg n -> "non("^(string_of_prop n)^")"
  | Implique (p1, p2) -> "("^string_of_prop p1^") => ("^(string_of_prop p2)^")"
  | Et (p1, p2) -> "("^string_of_prop p1^") et ("^(string_of_prop p2)^")"
  | Ou (p1, p2) -> "("^string_of_prop p1^") ou ("^(string_of_prop p2)^")"
;;

type contexte = (string * bool) list;;

(*4*)
let rec test_contexte c =
  let rec pas_element (ss, bb) l = match l with 
    | [] -> true
    | (s, _)::q -> (s=ss) || pas_element (ss,bb) q
     in
    match c with
      | [] -> true
      | t::q -> pas_element t q &&  test_contexte q
;;

(*5*)
let rec evalue p cont =
  let rec test_var s c = match c with
    | [] -> failwith "Contexte insuffisant"
    | (ss,bb)::fc -> if s=ss then bb else test_var s fc in
    match prop with
      | Var s -> test_var s cont
      | Vrai -> true
      | Faux -> false
      | Neg p -> not (evalue p cont)
      | Implique (p, q) -> (evalue q cont) || (not (evalue p cont))
      | Et (p, q) -> (evalue p cont) && (evalue q cont)
      | Ou (p, q) -> (evalue p cont) || (evalue q cont)
;;

evalue prop_ex ex_contexte;;

let rec appartient x = function
    | [] -> false
    | y::fl -> if x=y then true else appartient x fl;;


let ajoute_elem x l =
    if appartient x l then l else x::l
;;

let union l1 l2 =
  let rec aux l1 l2 =
    match l1 with
      | [] -> l2
      | x::q1 -> if appartient x l2 then aux q1 l2 else x::(aux q1 l2);
  in aux l1 l2
;;

union [3; 7; 2; 1; 5] [7; 8; 1; 9; 3];;


type clause = proposition list;;
type forme_clausale = clause list;;

let rec prod (fc1 : forme_clausale) (fc2 : forme_clausale) =
	let rec aux (cl : clause) (cl_list : forme_clausale) = match cl_list with
			| [] -> []
			| cl2 :: q -> ajoute_elem (union cl cl2) (aux cl q)
	in
		match fc1 with
			| [] -> fc2
			| c :: q -> union (aux c fc2) (prod q fc2);;


let rec prod ll1 ll2 =
	let rec distribue l1 ll3 =
		match ll3 with
			| [] -> []
			| l2 :: q -> ajoute_elem (union l1 l2)  (distribue l1 q)
	in
		match ll1 with
			| [] -> []
			| l1 :: q -> union (distribue l1 ll2) (prod q ll2);; 
			
	prod [[1;2];[3;4]] [[1;2];[3;8];[7;9];[10;11;12]];;


let rec fnc prop = match prop with
    | Var s -> [[Var s]]
    | Neg (Var s) -> [[Neg (Var s)]]
    | Faux -> [[Faux]]
    | Neg Faux -> [[Neg Faux]]
    | Et (p, q) -> union (fnc p) (fnc q)
    | Ou (p, q) -> prod (fnc p) (fnc q)
    | Implique (p, q) -> prod (fnc (Neg p)) (fnc q)
    | Vrai -> [[Neg Faux]]
    | Neg Vrai -> [[Faux]]
    | Neg (Et (p, q)) -> prod (fnc (Neg p)) (fnc (Neg q))
    | Neg (Ou (p, q)) -> union (fnc (Neg p)) (fnc (Neg q))
    | Neg (Implique (p, q)) -> union (fnc p) (fnc (Neg q))
    | Neg (Neg p) -> fnc p;;


let va = Var "A";;
let vb = Var "B";;
let p = Implique(Et(va, vb), va) in fnc p;;
let q = Implique (va,Et(va,vb)) in fnc q;;
let r = Implique (va, Implique(Implique(va,vb) ,vb)) in fnc r;;

let rec enleve x l = match l with
    | [] -> []
    | t :: q when t = x -> enleve x q
    | t :: q -> t :: (enleve x q);;


let rec resolution c1 c2 =
  let rec aux cl1 = match cl1 with
      | [] -> []
      | Var s :: q -> if appartient (Neg (Var s)) c2 then (union (enleve (Var s) c1) (enleve (Neg (Var s)) c2)) :: (aux q)
          else aux q
      | Neg (Var s) :: q -> if appartient (Var s) c2 then (union (enleve (Neg (Var s)) c1) (enleve (Var s) c2)) :: (aux q)
          else aux q
      | Faux :: q -> if appartient (Neg (Faux)) c2 then (union (enleve (Faux) c1) (enleve (Neg (Faux)) c2)) :: (aux q)
          else aux q
      | Neg (Faux) :: q -> if appartient (Faux) c2 then (union (enleve (Neg (Faux)) c1) (enleve (Faux) c2)) :: (aux q)
          else aux q
 in aux c1;;
  


let cl1 = [Var "P" ; Neg(Var"Q")];;
let cl2 = [Var "Q" ; Var "R" ; Neg(Var"P")];;
let cl3 = [Var "P"];;
let cl4 = [Neg(Var "P")];;
resolution cl3 cl4;;

let rec succes li = match li with
    | [] -> false
    | t :: q -> (t = []) || (t=[Faux]) || (succes q);; 



let rec couples l = match l with 
|[] -> []
| t:: q -> let rec accouple x li = match li with 
	|[] -> []
	|t1::q1 -> if t1 = x then accouple x q1 else (x,t1) :: (accouple x q1)
	in union (accouple t q) (couples q);;


(* comme il n'y a pas de r�p�titon dans la liste des clauses l on n'obtient pas des r�p�titions du type (x,y) puis (y,x)  *)

let decision p =
  let rec procedure list_cause =
    let rec appli l = match l with
        | [] -> false
        | (a, b) :: q -> let r = resolution a b in
              let rec appli_proc li = match li with
                  | [] -> false
                  | t :: q -> (procedure (t::q)) || appli_proc q
              in
                (succes r) || appli_proc r
    in appli (couples list_cause)
  in
    procedure (fnc (Neg (p)));;
		 


decision p;;

